
uniform mat2 frontScale;
uniform vec2 frontTranslate;

uniform mat2 backScale;
uniform vec2 backTranslate;

uniform float zoomAmount;

uniform float t;

uniform bool frontFlipped;

attribute vec4 position;
attribute vec2 texcoord;

varying vec2 v_texcoord;
varying float shadow;


float
bump(float p, float x, float width)
{
	x = (x - p) / width;

	if ( x < -1.0 || x > 1.0 )
		return 0.0;
	else
		return (cos(3.1415926535 * x) + 1.0) / 2.0;
}

void main()
{
	vec4 startPos = position;
	
	// curl the bottom
	if ( position.y < -0.4 )
		startPos.x += (position.y + 0.4) * (position.y + 0.4) / 2.0;
	
	if ( frontFlipped )
		startPos.y = -startPos.y;
	
	v_texcoord.xy = vec2(texcoord.x, 1.0 - texcoord.y);

	startPos.xy = startPos.xy * frontScale; // + frontTranslate;

	startPos.z = -0.1; // front side is underneath (larger z is farther away, remember?) the back

	vec4 endPos = position;
	
	endPos.xy = vec2(position.x, -position.y);
	endPos.xy = endPos.xy * backScale; // + backTranslate;
	endPos.z = -0.2;
	
	// fold position follows time by a bit, so that the page moves a bit before we flip it
	float foldlag = 0.33333;
	float fold = min(1.0, max(0.0, (1.0 + foldlag) * t - foldlag));
	
	// try and get a shadow where it's folding. this value goes out to the fragment shader.
	float s = (1.0 - fold) * bump(fold + 0.1, (position.y + 1.0) / 2.0, 0.2);
	shadow = 1.0 - s / 8.0;
	
	// interpolate between start and end positions, varying by y coordinate
	float u;
	
	// "smoothstep performs smooth Hermite interpolation between 0 and 1 when edge0 < x < edge1."
	// For position.y = -1 (bottom of the page), the function interpolates from 0 to 1 over the input range (0, 0.5)
	// For position.y = 1 (top of the page), the function interpolates from 0 to 1 over the input range (0.5, 1.0)
	
	u = smoothstep(position.y / 4.0 + 0.25, position.y / 4.0 + 0.75, fold);

	// also zoom
	
	float zoomTime = 0.5;
	float zoom = 1.0 + zoomAmount * (t < zoomTime ? 2.0 * t : 2.0 * zoomTime);
	
	startPos.xy *= zoom;

	// Dip the bottom down a little as it flips
	startPos.y -= 0.75 * u * (1.0 - position.y);
	
	// And move it from start to finish, on a curve
	float dx = frontTranslate.x + (backTranslate.x - frontTranslate.x) * smoothstep(0.0, 1.0, t);
	float dy = frontTranslate.y + (backTranslate.y - frontTranslate.y) * smoothstep(0.0, 1.0, sqrt(t));
	
    gl_Position = (1.0 - u) * startPos + u * endPos + vec4(dx, dy, 0.0, 0.0);
}
